unit Profile;

{----------------------------------------------------------------------------
Version 1.1  Mar/19/1997
(C) 1996 S. Armstrong
(C) 1997 Christoph R. Kirchner


Description:

  "Profile" encapsulates .INI files.
  I found it on the DSP and used it extensive. Later I wanted to
  use the registry instead of INI-files without changing all my
  projects.

  TProfileFile
    The INI-file specified by property Filename is used.
    Win32:
      If property ProfileSource = psRegistry then the
      Registry-path specified by property Filename is used.

  TLocalProfileFile
    It is called "local" because on the network there is one INI-file
    on each client.
    Win16:
      FileName = C:\WINDOWS\ExeFileName.INI
    Win32:
      ProfileSource = psRegistry:
        Registry-path = Software\ExeFileName
      else FileName = C:\WINDOWS\ExeFileName.INI

  TGobalProfileFile
    F:\ExePath\ExeFileName.INI
    Use this if your application gets installed on a network-drive
    and all users should use the same INI-file.
    No changes if used with Win32 - there is no global registry.

  TProfileSection
    property Section:
      Win16: [Section]
      Win32: ProfileFile.FileName\Section

  TProfileString

  TProfileInteger

  TProfileBoolean
    property BooleanType specities how values get written
    into the INI-file:
      bt01: Integer 1, 0
      btYesNo: String 'Yes', 'No'
      btTrueFalse: String 'True', 'False'


Tips:

  If you want to convert or copy your profile-entries, you can use
  a procedure like this:

  procedure TForm1.ConvertProfile(Idents: array of TProfileIdent);
  var
    i: Integer;
  begin
  (* read all values into internal buffers: *)
    for i := 0 to High(Idents) do Idents[i].ReadValue;
  (* change the location of ProfileFile, e.g.: *)
    LocalProfileFile1.ProfileSource := psRegistry;
  (* flush all values from internal buffers: *)
    for i := 0 to High(Idents) do Idents[i].ForceWrite;
  end;

  Call it like this: ConvertProfile([ProfileString1, ProfileString2]);


Author:

  Originally from:
    S. Armstrong     13/1/96
    100717.3531@compuserve.com
    Freeware.  Use it at your own risk.
  Changed by:
    Christoph R. Kirchner.
    ckirchner@geocities.com
    Still Freeware. Still your own risk.


P.S.:

  You can find my collection of components at:
  http://www.geocities.com/SiliconValley/Heights/7874/delphi.htm
  Maybe you can find an update of these components there.
  Also, there are components that are currently under construction -
  if one or more of them are useful for you:
  You can help me beta-testing them...

-----------------------------------------------------------------------------}

interface

uses
  WINPROCS, Classes, IniFiles, Forms, SysUtils
{$IFDEF WIN32}
  , Registry
{$ENDIF DEF WIN32}
  ;

const
  RegisterOnPage = 'Profile';

type
  TInvalidIdentException = class(Exception);
  TProfileSource = (psIniFile, psRegistry);

  TCustomProfileFile = class(TComponent)
  private
  {$IFDEF WIN32}
    FRegIniFile: TRegIniFile;
  {$ENDIF DEF WIN32}
    FIniFile: TIniFile;
    FFilename: TFileName;
    FProfileSource: TProfileSource;
    procedure SetFilename(fn: TFileName);
  protected
    procedure SetProfileSource(Value: TProfileSource); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor destroy; override;
    function ReadString(const Section, Ident, Default: String): String;
    procedure WriteString(const Section, Ident, Value: String);
    function ReadInteger(const Section, Ident: String;
      Default: Longint): Longint;
    procedure WriteInteger(const Section, Ident: String; Value: Longint);
    function ReadBool(const Section, Ident: String;
      Default: Boolean): Boolean;
    procedure WriteBool(const Section, Ident: String; Value: Boolean);
    property Filename: TFileName read FFilename write SetFilename;
    property ProfileSource: TProfileSource
      read FProfileSource write SetProfileSource;
  end;

{ TLocalProfileFile:
  C:\WINDOWS\ExeFileName.INI or Registry-path Software\ExeFileName. }
  TLocalProfileFile = class(TCustomProfileFile)
  public
    constructor Create(AOwner: TComponent); override;
  protected
    procedure SetProfileSource(Value: TProfileSource); override;
  published
    property ProfileSource;
  end;

{ TGlobalProfileFile: F:\ExePath\ExeFileName.INI
  Use this if your application gets installed on a network-drive and
  all users should use the same INI-file. }
  TGlobalProfileFile = class(TCustomProfileFile)
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TProfileFile = class(TCustomProfileFile)
  published
    property Filename;
    property ProfileSource;
  end;

  TProfileSection = class(TComponent)
  private
    FProfileFile: TCustomProfileFile;
    FSection: String;
  protected
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  published
    property ProfileFile: TCustomProfileFile
      read FProfileFile write FProfileFile;
    property Section: String read FSection write FSection;
  end;

  TProfileIdent = class(TComponent)
  private
    FProfileSection: TProfileSection;
    FIdent: String;
  { FValidCache: If true, a TProfileXXX.GetValue does not call ReadValue: }
    FValidCache: Boolean;
    procedure SetProfileSection(ps: TProfileSection);
    procedure SetIdent(id: String);
  protected
    constructor Create(AOwner: TComponent); override;
    procedure SetName(const NewName: TComponentName); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    function  ReadInteger(const ident: String; const default: integer): integer;
    procedure WriteInteger(const ident: String; const value: integer);
    function  ReadBoolean(const ident: String; const default: Boolean): Boolean;
    procedure WriteBoolean(const ident: String; const value: Boolean);
    function  ReadString(const ident, default: String): String;
    procedure WriteString(const ident, value: String);
  public
    procedure ReadValue; virtual;
    procedure ForceWrite; virtual;
  published
    property ProfileSection: TProfileSection
      read FProfileSection write SetProfileSection;
    property Ident: String read FIdent write SetIdent;
  end;

  TProfileString = class(TProfileIdent)
  private
    FValue: String;
    FDefaultValue: String;
    procedure SetDefaultValue(Value: String);
    procedure SetValue(Value: String);
    function  GetValue: String;
  public
    procedure ReadValue; override;
    procedure ForceWrite; override;
    property Value: String read GetValue write SetValue;
  published
    property DefaultValue: String read FDefaultValue write SetDefaultValue;
  end;

  TProfileInteger = class(TProfileIdent)
  private
    FValue: integer;
    FDefaultValue: integer;
    procedure SetDefaultValue(Value: integer);
    procedure SetValue(Value: integer);
    function  GetValue: integer;
  public
    procedure ReadValue; override;
    procedure ForceWrite; override;
    property Value: integer read GetValue write SetValue;
  published
    property DefaultValue: integer read FDefaultValue write SetDefaultValue default 0;
  end;

  TBooleanType = (bt01, btYesNo, btTrueFalse);

  TProfileBoolean = class(TProfileIdent)
  private
    FValue: Boolean;
    FDefaultValue: Boolean;
    FBooleanType: TBooleanType;
    procedure SetDefaultValue(Value: Boolean);
    procedure SetValue(Value: Boolean);
    function  GetValue: Boolean;
  public
    procedure ReadValue; override;
    procedure ForceWrite; override;
    property Value: Boolean read GetValue write SetValue;
  published
    property DefaultValue: Boolean
      read FDefaultValue write SetDefaultValue default false;
    property BooleanType: TBooleanType
      read FBooleanType write FBooleanType default bt01;
  end;

procedure Register;

implementation

{******************************************************************************}

procedure Register;
begin
  RegisterComponents(RegisterOnPage,
    [TProfileFile, TLocalProfileFile, TGlobalProfileFile, TProfileSection,
     TProfileString, TProfileInteger, TProfileBoolean]);
end;


{ TCustomProfileFile **********************************************************}

constructor TCustomProfileFile.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFilename := '';
  FIniFile := nil;
{$IFDEF WIN32}
  FRegIniFile := nil;
  FProfileSource := psRegistry;
{$ELSE DEF WIN32}
  FProfileSource := psIniFile;
{$ENDIF DEF WIN32}
end;

destructor TCustomProfileFile.Destroy;
begin
  FIniFile.Free;
{$IFDEF WIN32}
  FRegIniFile.Free;
{$ENDIF DEF WIN32}
  inherited destroy;
end;

procedure TCustomProfileFile.SetFilename(fn: TFileName);
begin
{$IFDEF WIN32}
  if (FFilename <> fn) then
  begin
    FFilename := fn;
    FIniFile.Free;
    FRegIniFile.Free;
    if (FProfileSource = psIniFile) then
    begin
      if (FFilename = '') then
        FIniFile := nil
      else
        FIniFile := TIniFile.Create(FFilename);
      FRegIniFile := nil;
    end
    else
    begin
      if (FFilename = '') then
        FRegIniFile := nil
      else
        FRegIniFile := TRegIniFile.Create(FFilename);
      FIniFile := nil;
    end;
  end;
{$ELSE DEF WIN32}
  if (FFilename <> fn) then
  begin
    FFilename := fn;
    FIniFile.Free;
    if (FFilename = '') then
      FIniFile := nil
    else
      FIniFile := TIniFile.Create(FFilename);
  end;
{$ENDIF DEF WIN32}
end;

procedure TCustomProfileFile.SetProfileSource(Value: TProfileSource);
begin
{$IFDEF WIN32}
  if (FProfileSource <> Value) then
  begin
    if (Value = psIniFile) then
    begin
      FRegIniFile.Free;
      FRegIniFile := nil;
      if (FFilename <> '') then
        FIniFile := TIniFile.Create(FFilename);
    end
    else
    begin
      FIniFile.Free;
      FIniFile := nil;
      if (FFilename <> '') then
        FRegIniFile := TRegIniFile.Create(FFilename);
    end;
    FProfileSource := Value;
  end;
{$ELSE DEF WIN32}
  FProfileSource := Value;
{$ENDIF DEF WIN32}
end;

function TCustomProfileFile.ReadString(const Section, Ident,
  Default: String): String;
begin
{$IFDEF WIN32}
  if (FProfileSource = psIniFile) and Assigned(FIniFile) then
    result := FIniFile.ReadString(Section, Ident, Default)
  else
    if Assigned(FRegIniFile) then
      result := FRegIniFile.ReadString(Section, Ident, Default)
    else
      result := Default;
{$ELSE DEF WIN32}
  if Assigned(FIniFile) then
    result := FIniFile.ReadString(Section, Ident, Default)
  else
    result := Default;
{$ENDIF DEF WIN32}
end;

procedure TCustomProfileFile.WriteString(const Section, Ident, Value: String);
begin
{$IFDEF WIN32}
  if (FProfileSource = psIniFile) and Assigned(FIniFile) then
    FIniFile.WriteString(Section, Ident, Value)
  else
    if Assigned(FRegIniFile) then
      FRegIniFile.WriteString(Section, Ident, Value);
{$ELSE DEF WIN32}
  if Assigned(FIniFile) then
    FIniFile.WriteString(Section, Ident, Value);
{$ENDIF DEF WIN32}
end;

function TCustomProfileFile.ReadInteger(const Section, Ident: String;
  Default: Longint): Longint;
begin
{$IFDEF WIN32}
  if (FProfileSource = psIniFile) and Assigned(FIniFile) then
    result := FIniFile.ReadInteger(Section, Ident, Default)
  else
    if Assigned(FRegIniFile) then
      result := FRegIniFile.ReadInteger(Section, Ident, Default)
    else
      result := Default;
{$ELSE DEF WIN32}
  if Assigned(FIniFile) then
    result := FIniFile.ReadInteger(Section, Ident, Default)
  else
    result := Default;
{$ENDIF DEF WIN32}
end;

procedure TCustomProfileFile.WriteInteger(const Section, Ident: String;
  Value: Longint);
begin
{$IFDEF WIN32}
  if (FProfileSource = psIniFile) and Assigned(FIniFile) then
    FIniFile.WriteInteger(Section, Ident, Value)
  else
    if Assigned(FRegIniFile) then
      FRegIniFile.WriteInteger(Section, Ident, Value);
{$ELSE DEF WIN32}
  if Assigned(FIniFile) then
    FIniFile.WriteInteger(Section, Ident, Value);
{$ENDIF DEF WIN32}
end;

function TCustomProfileFile.ReadBool(const Section, Ident: String;
  Default: Boolean): Boolean;
begin
{$IFDEF WIN32}
  if (FProfileSource = psIniFile) and Assigned(FIniFile) then
    result := FIniFile.ReadBool(Section, Ident, Default)
  else
    if Assigned(FRegIniFile) then
      result := FRegIniFile.ReadBool(Section, Ident, Default)
    else
      result := Default;
{$ELSE DEF WIN32}
  if Assigned(FIniFile) then
    result := FIniFile.ReadBool(Section, Ident, Default)
  else
    result := Default;
{$ENDIF DEF WIN32}
end;

procedure TCustomProfileFile.WriteBool(const Section, Ident: String;
  Value: Boolean);
begin
{$IFDEF WIN32}
  if (FProfileSource = psIniFile) and Assigned(FIniFile) then
    FIniFile.WriteBool(Section, Ident, Value)
  else
    if Assigned(FRegIniFile) then
      FRegIniFile.WriteBool(Section, Ident, Value);
{$ELSE DEF WIN32}
  if Assigned(FIniFile) then
    FIniFile.WriteBool(Section, Ident, Value);
{$ENDIF DEF WIN32}
end;


{******************************************************************************}

constructor TLocalProfileFile.Create(AOwner: TComponent);
var
  winDir: array[0..255] of char;
begin
  inherited Create(AOwner);
  if not (csDesigning in ComponentState) then
  begin
(*  FProfileSource := psRegistry;
    SetFilename('Software\' +
      ChangeFileExt(ExtractFileName(Application.ExeName), ''));   *)
    FProfileSource := psIniFile;
    GetWindowsDirectory(winDir, 127);
    SetFilename(StrPas(winDir) + '\' +
      ChangeFileExt(ExtractFileName(Application.ExeName),'.INI'));
  end;
end;

procedure TLocalProfileFile.SetProfileSource(Value: TProfileSource);
{$IFDEF WIN32}
var
  winDir: array[0..255] of char;
{$ENDIF DEF WIN32}
begin
{$IFDEF WIN32}
  if (FProfileSource <> Value) then
  begin
    if (Value = psIniFile) then
    begin
      GetWindowsDirectory(winDir, 127);
      FFilename := (StrPas(winDir) + '\' +
        ChangeFileExt(ExtractFileName(Application.ExeName),'.INI'));
    end
    else
    begin
      FFilename := ('Software\' +
        ChangeFileExt(ExtractFileName(Application.ExeName), ''));
    end;
    inherited;
  end;
{$ELSE DEF WIN32}
  FProfileSource := Value;
{$ENDIF DEF WIN32}
end;


constructor TGlobalProfileFile.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
{$IFDEF WIN32}
  FProfileSource := psIniFile;
{$ENDIF DEF WIN32}
  if not (csDesigning in ComponentState) then
    SetFilename(ChangeFileExt(Application.ExeName,'.INI'));
end;

{******************************************************************************}

procedure TProfileSection.SetName(const NewName: TComponentName);
var
   same: Boolean;
begin
  same := FSection = Name;
  inherited SetName(NewName);
  if same then
    FSection := Name;
end;

procedure TProfileSection.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FProfileFile <> nil) and
     (AComponent = FProfileFile) then
    ProfileFile := nil;
end;

{ TProfileIdent ------------------------------------------------------------- }

constructor TProfileIdent.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FValidCache := false;
end;

procedure TProfileIdent.SetProfileSection(ps: TProfileSection);
begin
  if FProfileSection <> ps then
  begin
    FProfileSection := ps;
    if FValidCache then
      ReadValue;
  end;
end;

procedure TProfileIdent.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FProfileSection <> nil) and
     (AComponent = FProfileSection) then
    ProfileSection := nil;
end;

procedure TProfileIdent.SetIdent(id: String);
begin
  if FIdent <> id then
  begin
    if Pos(id,'=') > 0 then
      raise TInvalidIdentException.Create(id + ' is an invalid ident name')
    else
    begin
      FIdent := id;
      if FValidCache then
        ReadValue;
    end;
  end;
end;

procedure TProfileIdent.SetName(const NewName: TComponentName);
var
  same: Boolean;
begin
  same := FIdent = Name;
  inherited SetName(NewName);
  if same then
    FIdent := Name;
end;

procedure TProfileIdent.ReadValue;
begin
{ allow overriding in descendents }
end;

function TProfileIdent.ReadInteger(const ident: String;
  const default: integer): integer;
begin
  if (ProfileSection <> nil) and (ProfileSection.ProfileFile <> nil) then
    result := ProfileSection.ProfileFile.ReadInteger(
      ProfileSection.section, Ident, Default)
  else
    result := default;
end;

procedure TProfileIdent.WriteInteger(const ident: String; const value: integer);
begin
  if (ProfileSection <> nil) and (ProfileSection.ProfileFile <> nil) then
    ProfileSection.ProfileFile.WriteInteger(
      ProfileSection.Section, Ident, Value);
end;

function TProfileIdent.ReadBoolean(const ident: String;
  const default: Boolean): Boolean;
begin
  if (ProfileSection <> nil) and (ProfileSection.ProfileFile <> nil) then
    result := ProfileSection.ProfileFile.ReadBool(
      ProfileSection.section, Ident, Default)
  else
    result := default;
end;

procedure TProfileIdent.WriteBoolean(const ident: String; const value: Boolean);
begin
  if (ProfileSection <> nil) and (ProfileSection.ProfileFile <> nil) then
    ProfileSection.ProfileFile.WriteBool(ProfileSection.Section, Ident, Value);
end;

function TProfileIdent.ReadString(const ident, default: String): String;
begin
  if (ProfileSection <> nil) and (ProfileSection.ProfileFile <> nil) then
    result := ProfileSection.ProfileFile.ReadString(
      ProfileSection.section, Ident, Default)
  else
    result := default;
end;

procedure TProfileIdent.WriteString(const ident, value: String);
begin
  if (ProfileSection <> nil) and (ProfileSection.ProfileFile <> nil) then
    ProfileSection.ProfileFile.WriteString(
      ProfileSection.Section, Ident, Value);
end;

procedure TProfileIdent.ForceWrite;
begin
  if not FValidCache then
    raise Exception.Create(
      'TProfileIdent: ForceWrite called but cache is invalid');
  FValidCache := false;
end;


{ TProfileString ------------------------------------------------------------ }

procedure TProfileString.SetDefaultValue(Value: String);
begin
  if FDefaultValue <> Value then
  begin
    FDefaultValue := Value;
    if FValidCache then
      ReadValue;
  end;
end;

procedure TProfileString.SetValue(Value: String);
begin
  if (FValue <> Value) or not FValidCache then
  begin
    FValue := Value;
    WriteString(Ident, Value);
    FValidCache := true;
  end
  else
    if FDefaultValue = Value then
      WriteString(Ident, Value);
end;

function TProfileString.GetValue: String;
begin
  if not FValidCache then
    ReadValue;
  result := FValue;
end;

procedure TProfileString.ReadValue;
begin
  if not (csLoading in ComponentState) then
  begin
    FValue := ReadString(Ident, FDefaultValue);
    FValidCache := true;
  end;
end;

procedure TProfileString.ForceWrite;
begin
  inherited ForceWrite;
  SetValue(FValue);
end;


{ TProfileInteger ----------------------------------------------------------- }

procedure TProfileInteger.SetDefaultValue(Value: integer);
begin
  if FDefaultValue <> Value then
  begin
    FDefaultValue := Value;
    if FValidCache then
      ReadValue;
  end;
end;

function TProfileInteger.GetValue: Integer;
begin
  if not FValidCache then
    ReadValue;
  result := FValue;
end;

procedure TProfileInteger.SetValue(Value: integer);
begin
  if (FValue <> Value) or not FValidCache then
  begin
    FValue := Value;
    WriteInteger(Ident, Value);
    FValidCache := true;
  end
  else
    if FDefaultValue = Value then
      WriteInteger(Ident, Value);
end;

procedure TProfileInteger.ReadValue;
begin
  if not (csLoading in ComponentState) then
  begin
    FValue := ReadInteger(Ident, FDefaultValue);
    FValidCache := true;
  end;
end;

procedure TProfileInteger.ForceWrite;
begin
  inherited ForceWrite;
  SetValue(FValue);
end;


{ TProfileBoolean ----------------------------------------------------------- }

procedure TProfileBoolean.SetDefaultValue(Value: Boolean);
begin
  if FDefaultValue <> Value then
  begin
    FDefaultValue := Value;
    if FValidCache then
      ReadValue;
  end;
end;

const
  BooleanTypes: array[TBooleanType, Boolean] of String[7] =
    (('0', '1'), ('No', 'Yes'), ('False', 'True'));

procedure TProfileBoolean.SetValue(Value: Boolean);
begin
  if (FValue <> Value) or not FValidCache then
  begin
    FValue := Value;
    if (FBooleanType = bt01) then
      WriteBoolean(Ident, Value)
    else
      WriteString(Ident, BooleanTypes[FBooleanType, Value]);
    FValidCache := true;
  end
  else
    if FDefaultValue = Value then
      if (FBooleanType = bt01) then
        WriteBoolean(Ident, Value)
      else
        WriteString(Ident, BooleanTypes[FBooleanType, Value]);
end;

function TProfileBoolean.GetValue: Boolean;
begin
  if not FValidCache then
    ReadValue;
  result := FValue;
end;

procedure TProfileBoolean.ReadValue;
var
  s: String;
begin
  if not (csLoading in ComponentState) then
  begin
    s := UpperCase(ReadString(Ident,
                              BooleanTypes[FBooleanType, FDefaultValue]));
    FValue := (s = BooleanTypes[bt01, true]) or
              (s = UpperCase(BooleanTypes[btYesNo, true])) or
              (s = UpperCase(BooleanTypes[btTrueFalse, true]));
    FValidCache := true;
  end;
end;

procedure TProfileBoolean.ForceWrite;
begin
  inherited ForceWrite;
  SetValue(FValue);
end;

end.
